@@ -261,44 +261,101 @@ main(int argc, char *argv[])
 				*din = NULL;
 	const char		*fnin = "<stdin>", *fnout = NULL,
 	      	 		*fndin = NULL, *extract = NULL, *er,
-				*mainopts = "LM:m:sT:t:o:X:",
-	      			*diffopts = "M:m:sT:t:o:",
+				*mainopts = "LM:m:sT:t:o:X:h",
+				*diffopts = "M:m:sT:t:o:h",
 				*templfn = NULL, *odtstylefn = NULL;
+	struct lowdown_opts_term topts;
 	struct lowdown_opts 	 opts;
-	int			 c, diff = 0, status = 0, afl = 0,
-				 rfl = 0, aifl = 0, rifl = 0,
-				 centre = 0, list = 0;
+	int			 c, diff = 0, status = 1, afl = 0,
+				 rfl = 0, aifl = 0, rifl = 0, list = 0;
 	char			*ret = NULL, *cp, *templptr = NULL,
 				*nroffcodefn = NULL,
 				*odtstyleptr = NULL;
-	size_t		 	 i, retsz = 0, rcols;
+	size_t		 	 i, retsz = 0;
 	struct lowdown_meta 	*m;
 	struct lowdown_metaq	 mq;
 	struct option 		 lo[] = {
-		{ "html-escapehtml",	no_argument,	&afl, LOWDOWN_HTML_ESCAPE },
+		{ "template",		required_argument, NULL, 8 },
+		{ "version",		no_argument,	NULL, 10 },
+		{ "help",		no_argument,	NULL, 11 },
+		{ "out-no-smarty",	no_argument,	&rfl, LOWDOWN_SMARTY },
+		{ "out-standalone",	no_argument,	&afl, LOWDOWN_STANDALONE },
+
 		{ "html-no-escapehtml",	no_argument,	&rfl, LOWDOWN_HTML_ESCAPE },
 		{ "html-hardwrap",	no_argument,	&afl, LOWDOWN_HTML_HARD_WRAP },
-		{ "html-no-hardwrap",	no_argument,	&rfl, LOWDOWN_HTML_HARD_WRAP },
-		{ "html-head-ids",	no_argument,	&afl, LOWDOWN_HTML_HEAD_IDS },
 		{ "html-no-head-ids",	no_argument,	&rfl, LOWDOWN_HTML_HEAD_IDS },
-		{ "html-num-ent",	no_argument,	&afl, LOWDOWN_HTML_NUM_ENT },
 		{ "html-no-num-ent",	no_argument,	&rfl, LOWDOWN_HTML_NUM_ENT },
-		{ "html-owasp",		no_argument,	&afl, LOWDOWN_HTML_OWASP },
 		{ "html-no-owasp",	no_argument,	&rfl, LOWDOWN_HTML_OWASP },
-		{ "html-skiphtml",	no_argument,	&afl, LOWDOWN_HTML_SKIP_HTML },
 		{ "html-no-skiphtml",	no_argument,	&rfl, LOWDOWN_HTML_SKIP_HTML },
 		{ "html-titleblock",	no_argument,	&afl, LOWDOWN_HTML_TITLEBLOCK },
-		{ "html-no-titleblock",	no_argument,	&rfl, LOWDOWN_HTML_TITLEBLOCK },
 		{ "html-callout-mdn",	no_argument,	&afl, LOWDOWN_HTML_CALLOUT_MDN },
-		{ "html-no-callout-mdn",no_argument,	&rfl, LOWDOWN_HTML_CALLOUT_MDN },
 		{ "html-callout-gfm",	no_argument,	&afl, LOWDOWN_HTML_CALLOUT_GFM },
-		{ "html-no-callout-gfm",no_argument,	&rfl, LOWDOWN_HTML_CALLOUT_GFM },
 
-		{ "latex-numbered",	no_argument,	&afl, LOWDOWN_LATEX_NUMBERED },
 		{ "latex-no-numbered",	no_argument,	&rfl, LOWDOWN_LATEX_NUMBERED },
-		{ "latex-skiphtml",	no_argument,	&afl, LOWDOWN_LATEX_SKIP_HTML },
 		{ "latex-no-skiphtml",	no_argument,	&rfl, LOWDOWN_LATEX_SKIP_HTML },
 
+		{ "roff-traditional",	no_argument,	&rfl, LOWDOWN_NROFF_GROFF },
+		{ "roff-no-links",	no_argument, 	&afl, LOWDOWN_NROFF_NOLINK },
+		{ "roff-no-numbered",	no_argument,	&rfl, LOWDOWN_NROFF_NUMBERED },
+		{ "roff-short-links",	no_argument, 	&afl, LOWDOWN_NROFF_SHORTLINK },
+		{ "roff-no-skiphtml",	no_argument,	&rfl, LOWDOWN_NROFF_SKIP_HTML },
+		{ "roff-endnotes",	no_argument,	&afl, LOWDOWN_NROFF_ENDNOTES },
+		{ "roff-code-font",	required_argument, NULL, 7 },
+
+		{ "odt-no-skiphtml",	no_argument,	&rfl, LOWDOWN_ODT_SKIP_HTML },
+		{ "odt-style",		required_argument, NULL, 6 },
+
+		{ "gemini-link-end",	no_argument, 	&afl, LOWDOWN_GEMINI_LINK_END },
+		{ "gemini-link-roman",	no_argument, 	&afl, LOWDOWN_GEMINI_LINK_ROMAN },
+		{ "gemini-link-noref",	no_argument, 	&afl, LOWDOWN_GEMINI_LINK_NOREF },
+		{ "gemini-link-inline",	no_argument, 	&afl, LOWDOWN_GEMINI_LINK_IN },
+		{ "gemini-metadata",	no_argument, 	&afl, LOWDOWN_GEMINI_METADATA },
+
+		{ "term-no-ansi",	no_argument, 	&afl, LOWDOWN_TERM_NOANSI },
+		{ "term-no-colour",	no_argument, 	&afl, LOWDOWN_TERM_NOCOLOUR },
+		{ "term-no-rellinks",	no_argument, 	&afl, LOWDOWN_TERM_NORELLINK },
+		{ "term-no-links",	no_argument, 	&afl, LOWDOWN_TERM_NOLINK },
+		{ "term-short-links",	no_argument, 	&afl, LOWDOWN_TERM_SHORTLINK },
+		{ "term-all-meta",	no_argument, 	&afl, LOWDOWN_TERM_ALL_META },
+		{ "term-all-metadata",	no_argument, 	&afl, LOWDOWN_TERM_ALL_META },
+		{ "term-columns",	required_argument, NULL, 4 },
+		{ "term-hmargin",	required_argument, NULL, 2 },
+		{ "term-hpadding",	required_argument, NULL, 9 },
+		{ "term-vmargin",	required_argument, NULL, 3 },
+		{ "term-width",		required_argument, NULL, 1 },
+
+		{ "parse-hilite",	no_argument,	&aifl, LOWDOWN_HILITE },
+		{ "parse-no-tables",	no_argument,	&rifl, LOWDOWN_TABLES },
+		{ "parse-no-fenced",	no_argument,	&rifl, LOWDOWN_FENCED },
+		{ "parse-no-footnotes",	no_argument,	&rifl, LOWDOWN_FOOTNOTES },
+		{ "parse-no-autolink",	no_argument,	&rifl, LOWDOWN_AUTOLINK },
+		{ "parse-no-strike",	no_argument,	&rifl, LOWDOWN_STRIKE },
+		{ "parse-no-super",	no_argument,	&rifl, LOWDOWN_SUPER },
+		{ "parse-super-short",	no_argument,	&aifl, LOWDOWN_SUPER_SHORT },
+		{ "parse-math",		no_argument,	&aifl, LOWDOWN_MATH },
+		{ "parse-no-mantitle",	no_argument,	&rifl, LOWDOWN_MANTITLE },
+		{ "parse-no-codeindent",no_argument,	&aifl, LOWDOWN_NOCODEIND },
+		{ "parse-no-intraemph",	no_argument,	&aifl, LOWDOWN_NOINTEM },
+		{ "parse-no-metadata",	no_argument,	&rifl, LOWDOWN_METADATA },
+		{ "parse-no-cmark",	no_argument,	&rifl, LOWDOWN_COMMONMARK },
+		{ "parse-no-deflists",	no_argument,	&rifl, LOWDOWN_DEFLIST },
+		/* TODO: remove these... */
+		{ "parse-no-img-ext",	no_argument,	&rifl, LOWDOWN_IMG_EXT },
+		{ "parse-no-ext-attrs",	no_argument,	&rifl, LOWDOWN_ATTRS },
+		{ "parse-no-tasklists",	no_argument,	&rifl, LOWDOWN_TASKLIST },
+		{ "parse-no-callouts",	no_argument,	&rifl, LOWDOWN_CALLOUTS },
+		{ "parse-maxdepth",	required_argument, NULL, 5 },
+
+		/*
+		 * What follows are options that are deprecated.  These
+		 * are still accepted, but are not documented.
+		 */
+
+		/*
+		 * Deprecated "nroff" prefix: lowdown moved from
+		 * --nroff-xyz to --roff-xyz in 1.4.0.
+		 */
+
 		{ "nroff-traditional",	no_argument,	&rfl, LOWDOWN_NROFF_GROFF },
 		{ "nroff-no-traditional",no_argument,	&afl, LOWDOWN_NROFF_GROFF },
 		{ "nroff-no-groff",	no_argument,	&rfl, LOWDOWN_NROFF_GROFF },
@@ -315,92 +372,72 @@ main(int argc, char *argv[])
 		{ "nroff-no-endnotes",	no_argument,	&rfl, LOWDOWN_NROFF_ENDNOTES },
 		{ "nroff-code-font",	required_argument, NULL, 7 },
 
-		{ "odt-skiphtml",	no_argument,	&afl, LOWDOWN_ODT_SKIP_HTML },
-		{ "odt-no-skiphtml",	no_argument,	&rfl, LOWDOWN_ODT_SKIP_HTML },
-		{ "odt-style",		required_argument, NULL, 6 },
+		/*
+		 * Deprecated "inverse" options: these were never
+		 * documented to start with.  They turn back on
+		 * functionality that's the default to start with.
+		 */
 
-		{ "term-columns",	required_argument, NULL, 4 },
-		{ "term-hmargin",	required_argument, NULL, 2 },
-		{ "term-vmargin",	required_argument, NULL, 3 },
-		{ "term-width",		required_argument, NULL, 1 },
-
-		{ "gemini-link-end",	no_argument, 	&afl, LOWDOWN_GEMINI_LINK_END },
-		{ "gemini-no-link-end",	no_argument, 	&rfl, LOWDOWN_GEMINI_LINK_END },
-		{ "gemini-link-roman",	no_argument, 	&afl, LOWDOWN_GEMINI_LINK_ROMAN },
-		{ "gemini-no-link-roman", no_argument, 	&rfl, LOWDOWN_GEMINI_LINK_ROMAN },
-		{ "gemini-link-noref",	no_argument, 	&afl, LOWDOWN_GEMINI_LINK_NOREF },
-		{ "gemini-no-link-noref", no_argument, 	&rfl, LOWDOWN_GEMINI_LINK_NOREF },
-		{ "gemini-link-inline",	no_argument, 	&afl, LOWDOWN_GEMINI_LINK_IN },
-		{ "gemini-no-link-inline",no_argument, 	&rfl, LOWDOWN_GEMINI_LINK_IN },
-		{ "gemini-metadata",	no_argument, 	&afl, LOWDOWN_GEMINI_METADATA },
-		{ "gemini-no-metadata",	no_argument, 	&rfl, LOWDOWN_GEMINI_METADATA },
-
-		{ "term-no-ansi",	no_argument, 	&afl, LOWDOWN_TERM_NOANSI },
-		{ "term-ansi",		no_argument, 	&rfl, LOWDOWN_TERM_NOANSI },
-		{ "term-no-colour",	no_argument, 	&afl, LOWDOWN_TERM_NOCOLOUR },
 		{ "term-colour",	no_argument, 	&rfl, LOWDOWN_TERM_NOCOLOUR },
-		{ "term-nolinks",	no_argument, 	&afl, LOWDOWN_TERM_NOLINK },
-		{ "term-no-nolinks",	no_argument, 	&rfl, LOWDOWN_TERM_NOLINK },
-		{ "term-shortlinks",	no_argument, 	&afl, LOWDOWN_TERM_SHORTLINK },
-		{ "term-no-shortlinks",	no_argument, 	&rfl, LOWDOWN_TERM_SHORTLINK },
-		{ "term-all-meta",	no_argument, 	&afl, LOWDOWN_TERM_ALL_META },
-		{ "term-all-metadata",	no_argument, 	&afl, LOWDOWN_TERM_ALL_META },
+		{ "term-ansi",		no_argument, 	&rfl, LOWDOWN_TERM_NOANSI },
 		{ "term-no-all-meta",	no_argument, 	&rfl, LOWDOWN_TERM_ALL_META },
 		{ "term-no-all-metadata", no_argument, 	&rfl, LOWDOWN_TERM_ALL_META },
-
+		{ "term-no-nolinks",	no_argument, 	&rfl, LOWDOWN_TERM_NOLINK },
+		{ "term-no-shortlinks",	no_argument, 	&rfl, LOWDOWN_TERM_SHORTLINK },
 		{ "out-smarty",		no_argument,	&afl, LOWDOWN_SMARTY },
-		{ "out-no-smarty",	no_argument,	&rfl, LOWDOWN_SMARTY },
-		{ "out-standalone",	no_argument,	&afl, LOWDOWN_STANDALONE },
 		{ "out-no-standalone",	no_argument,	&rfl, LOWDOWN_STANDALONE },
-
-		{ "parse-hilite",	no_argument,	&aifl, LOWDOWN_HILITE },
+		{ "gemini-no-link-inline",no_argument, 	&rfl, LOWDOWN_GEMINI_LINK_IN },
+		{ "gemini-no-link-end",	no_argument, 	&rfl, LOWDOWN_GEMINI_LINK_END },
+		{ "gemini-no-link-roman", no_argument, 	&rfl, LOWDOWN_GEMINI_LINK_ROMAN },
+		{ "gemini-no-link-noref", no_argument, 	&rfl, LOWDOWN_GEMINI_LINK_NOREF },
+		{ "gemini-no-metadata",	no_argument, 	&rfl, LOWDOWN_GEMINI_METADATA },
+		{ "odt-skiphtml",	no_argument,	&afl, LOWDOWN_ODT_SKIP_HTML },
+		{ "latex-numbered",	no_argument,	&afl, LOWDOWN_LATEX_NUMBERED },
+		{ "latex-skiphtml",	no_argument,	&afl, LOWDOWN_LATEX_SKIP_HTML },
+		{ "html-escapehtml",	no_argument,	&afl, LOWDOWN_HTML_ESCAPE },
+		{ "html-no-hardwrap",	no_argument,	&rfl, LOWDOWN_HTML_HARD_WRAP },
+		{ "html-head-ids",	no_argument,	&afl, LOWDOWN_HTML_HEAD_IDS },
+		{ "html-num-ent",	no_argument,	&afl, LOWDOWN_HTML_NUM_ENT },
+		{ "html-owasp",		no_argument,	&afl, LOWDOWN_HTML_OWASP },
+		{ "html-skiphtml",	no_argument,	&afl, LOWDOWN_HTML_SKIP_HTML },
+		{ "html-no-titleblock",	no_argument,	&rfl, LOWDOWN_HTML_TITLEBLOCK },
+		{ "html-no-callout-mdn",no_argument,	&rfl, LOWDOWN_HTML_CALLOUT_MDN },
+		{ "html-no-callout-gfm",no_argument,	&rfl, LOWDOWN_HTML_CALLOUT_GFM },
 		{ "parse-no-hilite",	no_argument,	&rifl, LOWDOWN_HILITE },
 		{ "parse-tables",	no_argument,	&aifl, LOWDOWN_TABLES },
-		{ "parse-no-tables",	no_argument,	&rifl, LOWDOWN_TABLES },
 		{ "parse-fenced",	no_argument,	&aifl, LOWDOWN_FENCED },
-		{ "parse-no-fenced",	no_argument,	&rifl, LOWDOWN_FENCED },
 		{ "parse-footnotes",	no_argument,	&aifl, LOWDOWN_FOOTNOTES },
-		{ "parse-no-footnotes",	no_argument,	&rifl, LOWDOWN_FOOTNOTES },
 		{ "parse-autolink",	no_argument,	&aifl, LOWDOWN_AUTOLINK },
-		{ "parse-no-autolink",	no_argument,	&rifl, LOWDOWN_AUTOLINK },
 		{ "parse-strike",	no_argument,	&aifl, LOWDOWN_STRIKE },
-		{ "parse-no-strike",	no_argument,	&rifl, LOWDOWN_STRIKE },
 		{ "parse-super",	no_argument,	&aifl, LOWDOWN_SUPER },
-		{ "parse-no-super",	no_argument,	&rifl, LOWDOWN_SUPER },
-		{ "parse-super-short",	no_argument,	&aifl, LOWDOWN_SUPER_SHORT },
 		{ "parse-no-super-short",no_argument,	&rifl, LOWDOWN_SUPER_SHORT },
-		{ "parse-math",		no_argument,	&aifl, LOWDOWN_MATH },
 		{ "parse-no-math",	no_argument,	&rifl, LOWDOWN_MATH },
 		{ "parse-mantitle",	no_argument,	&aifl, LOWDOWN_MANTITLE },
-		{ "parse-no-mantitle",	no_argument,	&rifl, LOWDOWN_MANTITLE },
 		{ "parse-codeindent",	no_argument,	&rifl, LOWDOWN_NOCODEIND },
-		{ "parse-no-codeindent",no_argument,	&aifl, LOWDOWN_NOCODEIND },
 		{ "parse-intraemph",	no_argument,	&rifl, LOWDOWN_NOINTEM },
-		{ "parse-no-intraemph",	no_argument,	&aifl, LOWDOWN_NOINTEM },
 		{ "parse-metadata",	no_argument,	&aifl, LOWDOWN_METADATA },
-		{ "parse-no-metadata",	no_argument,	&rifl, LOWDOWN_METADATA },
 		{ "parse-cmark",	no_argument,	&aifl, LOWDOWN_COMMONMARK },
-		{ "parse-no-cmark",	no_argument,	&rifl, LOWDOWN_COMMONMARK },
 		{ "parse-deflists",	no_argument,	&aifl, LOWDOWN_DEFLIST },
-		{ "parse-no-deflists",	no_argument,	&rifl, LOWDOWN_DEFLIST },
-		/* TODO: remove these... */
 		{ "parse-img-ext",	no_argument,	&aifl, LOWDOWN_IMG_EXT },
-		{ "parse-no-img-ext",	no_argument,	&rifl, LOWDOWN_IMG_EXT },
 		{ "parse-ext-attrs",	no_argument,	&aifl, LOWDOWN_ATTRS },
-		{ "parse-no-ext-attrs",	no_argument,	&rifl, LOWDOWN_ATTRS },
 		{ "parse-tasklists",	no_argument,	&aifl, LOWDOWN_TASKLIST },
-		{ "parse-no-tasklists",	no_argument,	&rifl, LOWDOWN_TASKLIST },
 		{ "parse-callouts",	no_argument,	&aifl, LOWDOWN_CALLOUTS },
-		{ "parse-no-callouts",	no_argument,	&rifl, LOWDOWN_CALLOUTS },
-		{ "parse-maxdepth",	required_argument, NULL, 5 },
 
-		{ "template",		required_argument, NULL, 8 },
+		/*
+		 * Some options re-hyphenated for consistency.
+		 */
+
+		{ "term-nolinks",	no_argument, 	&afl, LOWDOWN_TERM_NOLINK },
+		{ "term-shortlinks",	no_argument, 	&afl, LOWDOWN_TERM_SHORTLINK },
+
 		{ NULL,			0,	NULL,	0 }
 	};
 
 	/* Get the real number of columns or 72. */
 
-	rcols = get_columns();
+	memset(&topts, 0, sizeof(struct lowdown_opts_term));
+	topts.cols = get_columns();
+	topts.hpadding = 4;
 
 	sandbox_pre();
 
@@ -500,28 +537,29 @@ main(int argc, char *argv[])
 				opts.feat |= aifl;
 			break;
 		case 1:
-			opts.cols = strtonum(optarg, 0, INT_MAX, &er);
+			topts.width = strtonum(optarg, 0, INT_MAX, &er);
 			if (er == NULL)
 				break;
 			errx(1, "--term-width: %s", er);
 		case 2:
-			if (strcmp(optarg, "centre") == 0 ||
-			    strcmp(optarg, "centre") == 0) {
-				centre = 1;
+			if (strcmp(optarg, "auto") == 0 ||
+			    strcmp(optarg, "centre") == 0 ||
+			    strcmp(optarg, "center") == 0) {
+				topts.centre = 1;
 				break;
 			}
-			opts.hmargin = strtonum
+			topts.hmargin = strtonum
 				(optarg, 0, INT_MAX, &er);
 			if (er == NULL)
 				break;
 			errx(1, "--term-hmargin: %s", er);
 		case 3:
-			opts.vmargin = strtonum(optarg, 0, INT_MAX, &er);
+			topts.vmargin = strtonum(optarg, 0, INT_MAX, &er);
 			if (er == NULL)
 				break;
 			errx(1, "--term-vmargin: %s", er);
 		case 4:
-			rcols = strtonum(optarg, 1, INT_MAX, &er);
+			topts.cols = strtonum(optarg, 1, INT_MAX, &er);
 			if (er == NULL)
 				break;
 			errx(1, "--term-columns: %s", er);
@@ -552,10 +590,36 @@ main(int argc, char *argv[])
 		case 8:
 			templfn = optarg;
 			break;
+		case 9:
+			topts.hpadding = strtonum
+				(optarg, 0, INT_MAX, &er);
+			if (er == NULL)
+				break;
+			errx(1, "--term-hpadding: %s", er);
+		case 10:
+			/*
+			 * Print the version with --version.
+			 */
+			printf("lowdown %s\n", VERSION);
+			return 0;
+		case 'h':
+			/* FALLTHROUGH */
+		case 11:
+			/*
+			 * Used by -h or --help to print the usage
+			 * message and not exit with failure.
+			 */
+			status = 0;
+			goto usage;
 		default:
+			/*
+			 * Bad argument: exit with failure.
+			 */
 			goto usage;
 		}
 
+	status = 0;
+
 	argc -= optind;
 	argv += optind;
 
@@ -563,21 +627,8 @@ main(int argc, char *argv[])
  	    opts.type == LOWDOWN_GEMINI)
 		setlocale(LC_CTYPE, "");
 
-	/* 
-	 * By default, try to show 80 columns.
-	 * Don't show more than the number of available columns.
-	 */
-
-	if (opts.cols == 0) {
-		if ((opts.cols = rcols) > 80)
-			opts.cols = 80;
-	} else if (opts.cols > rcols)
-		opts.cols = rcols;
-
-	/* If we're centred, set our margins. */
-
-	if (centre && opts.cols < rcols)
-		opts.hmargin = (rcols - opts.cols) / 2;
+	if (opts.type == LOWDOWN_TERM)
+		opts.term = topts;
 
 	/* 
 	 * Diff mode takes two arguments: the first is mandatory (the
@@ -740,15 +791,15 @@ main(int argc, char *argv[])
 usage:
 	if (!diff) {
 		fprintf(stderr, 
-			"usage: lowdown [-Ls] [input_options] "
+			"usage: lowdown [-hLs] [input_options] "
 			"[output_options] [-M metadata]\n"
 			"               [-m metadata] "
 			"[-o output] [-t mode] [-X keyword] [file]\n");
 	} else
 		fprintf(stderr, 
-			"usage: lowdown-diff [-s] [input_options] "
+			"usage: lowdown-diff [-hs] [input_options] "
 			"[output_options] [-M metadata]\n"
 			"                    [-m metadata] "
 			"[-o output] [-t mode] oldfile [newfile]\n");
-	return 1;
+	return status;
 }
