@@ -43,8 +43,9 @@ struct term {
 	struct tstack		 *stack; /* stack of nodes */
 	size_t			  stackmax; /* size of stack */
 	size_t			  stackpos; /* position in stack */
-	size_t			  maxcol; /* soft limit */
+	size_t			  width; /* soft width of content */
 	size_t			  hmargin; /* left of content */
+	size_t			  hpadding; /* left of content */
 	size_t			  vmargin; /* before/after content */
 	struct lowdown_buf	 *tmp; /* for temporary allocations */
 	wchar_t			 *buf; /* buffer for counting wchar */
@@ -53,6 +54,7 @@ struct term {
 	size_t			  footsz; /* footnotes size  */
 	int			  footoff; /* don't collect (tables) */
 	struct lowdown_metaq	  metaq; /* metadata */
+	const struct lowdown_node*in_link; /* in an OSC8 hyperlink */
 };
 
 /*
@@ -171,6 +173,29 @@ rndr_mbswidth(struct term *term, const char *buf, size_t sz)
 	return csz == (size_t)-1 ? sz : csz;
 }
 
+/*
+ * Determine whether a link URL is relative.  Use a simple heuristic to
+ * accomplish this: a relative URL is one without a schema.  Returns
+ * zero if not a relative link, non-zero if it is.
+ */
+static int
+link_isrelative(const struct lowdown_buf *link)
+{
+	const char	*colon;
+	size_t	 	 rem;
+
+	/* If there's no colon, it's a relative link (no schema) */
+
+	if ((colon = memchr(link->data, ':', link->size)) == NULL)
+		return 1;
+
+	/* If there's a slash before the colon, it's a (rel) path. */
+
+	assert(colon > link->data);
+	rem = colon - link->data;
+	return memchr(link->data, '/', rem) != NULL;
+}
+
 /*
  * Copy the buffer into "out", escaping along the width.
  * Returns the number of actual printed columns, which in the case of
@@ -246,6 +271,58 @@ rndr_buf_unstyle(const struct term *term,
 	return HBUF_PUTSL(out, "\033[0m");
 }
 
+/*
+ * Start sequence for a terminal link (only in ANSI mode).  
+ */
+static int
+rndr_buf_osc8_open(const struct term *term, struct lowdown_buf *out,
+    const struct lowdown_node *n)
+{
+	const struct lowdown_buf	*uri = NULL;
+
+	if (term->opts & LOWDOWN_TERM_NOANSI)
+		return 1;
+	
+	if (n->type == LOWDOWN_LINK_AUTO)
+		uri = &n->rndr_autolink.link;
+	else if (n->type == LOWDOWN_LINK)
+		uri = &n->rndr_link.link;
+	else if (n->type == LOWDOWN_IMAGE)
+		uri = &n->rndr_image.link;
+
+	/*
+	 * Don't output an id for the link.  It's trivial to have a
+	 * random per-page value for this identifier (e.g., to
+	 * initialise a random number then append the node identifier),
+	 * but let the terminal handle this.
+	 */
+
+	assert(uri != NULL);
+	return HBUF_PUTSL(out, "\033]8;;") &&
+	    hbuf_putb(out, uri) &&
+	    HBUF_PUTSL(out, "\033\\");
+}
+
+/*
+ * Close a currently-open link.
+ */
+static int
+rndr_buf_osc8_close(const struct term *term, struct lowdown_buf *out)
+{
+
+	if (term->opts & LOWDOWN_TERM_NOANSI)
+		return 1;
+
+	/*
+	 * It would be trivial to crawl up our parent chain and either
+	 * switch from the current link or close out the link context
+	 * entirely, but lowdown(5) stipulates that nested links are not
+	 * possible.
+	 */
+
+	return HBUF_PUTSL(out, "\033]8;;\033\\");
+}
+
 /*
  * Output style "s" into "out" as an ANSI escape.  If "s" does not have
  * any style information or is NULL, output nothing.  Return zero on
@@ -411,7 +488,13 @@ static int
 rndr_buf_endwords(struct term *term, struct lowdown_buf *out,
 	const struct lowdown_node *n, const struct sty *osty)
 {
+	/*
+	 * If an OSC8 hyperlink should be closed, do it now (it doesn't
+	 * matter where this appears in relation to other styling).
+	 */
 
+	if (term->in_link && !rndr_buf_osc8_close(term, out))
+		return 0;
 	if (rndr_buf_endstyle(n))
 		return rndr_buf_unstyle(term, out, NULL);
 	if (osty != NULL)
@@ -522,21 +605,6 @@ rndr_buf_startline_prefixes(struct term *term,
 
 	emit = term->stack[i].lines++;
 
-	/*
-	 * If we're below the document root and not a header, that means
-	 * we're in a body part.  Emit the general body indentation.
-	 */
-
-	if (*depth == 0 && n->type != LOWDOWN_HEADER) {
-		if (!hbuf_puts(out, pfx_body.text))
-			return 0;
-		rndr_buf_advance(term, pfx_body.cols);
-	} else if (*depth == 0) {
-		if (!hbuf_puts(out, pfx_header.text))
-			return 0;
-		rndr_buf_advance(term, pfx_header.cols);
-	}
-
 	/*
 	 * Output any prefixes.
 	 * Any output must have rndr_buf_style() and set pstyle so that
@@ -560,6 +628,9 @@ rndr_buf_startline_prefixes(struct term *term,
 		for (i = 0; i < term->hmargin; i++)
 			if (!HBUF_PUTSL(out, " "))
 				return 0;
+		for (i = 0; i < term->hpadding; i++)
+			if (!HBUF_PUTSL(out, " "))
+				return 0;
 		break;
 	case LOWDOWN_BLOCKQUOTE:
 		rndr_node_style_apply(&sinner, &sty_bkqt_pfx);
@@ -685,9 +756,11 @@ rndr_buf_startline_prefixes(struct term *term,
 }
 
 /*
- * Like rndr_buf_startwords(), but at the start of a line.
- * This also outputs all line prefixes of the block context.
- * Return zero on failure (memory), non-zero on success.
+ * Like rndr_buf_startwords(), but at the start of a line.  (Unlike
+ * rndr_buf_endline(), which calls rndr_buf_endwords(), this does not
+ * call rndr_buf_startwords().)  This also outputs all line prefixes of
+ * the block context.  Return zero on failure (memory), non-zero on
+ * success.
  */
 static int
 rndr_buf_startline(struct term *term, struct lowdown_buf *out,
@@ -702,6 +775,16 @@ rndr_buf_startline(struct term *term, struct lowdown_buf *out,
 	memset(&s, 0, sizeof(struct sty));
 	if (!rndr_buf_startline_prefixes(term, &s, n, out, &depth))
 		return 0;
+
+	/*
+	 * If an OSC8 hyperlink should be printed, do it now (it doesn't
+	 * matter where this appears in relation to other styling).
+	 */
+
+	if (term->in_link != NULL &&
+	    !rndr_buf_osc8_open(term, out, term->in_link))
+		return 0;
+
 	if (osty != NULL)
 		rndr_node_style_apply(&s, osty);
 	return rndr_buf_style(term, out, &s);
@@ -767,6 +850,15 @@ rndr_buf_startwords(struct term *term, struct lowdown_buf *out,
 {
 	struct sty	 s;
 
+	/*
+	 * If an OSC8 hyperlink should be printed, do it now (it doesn't
+	 * matter where this appears in relation to other styling).
+	 */
+
+	if (term->in_link != NULL &&
+	    !rndr_buf_osc8_open(term, out, term->in_link))
+		return 0;
+
 	assert(!term->last_blank);
 	assert(term->col > 0);
 
@@ -782,8 +874,8 @@ rndr_buf_startwords(struct term *term, struct lowdown_buf *out,
  */
 static int
 rndr_buf_literal(struct term *term, struct lowdown_buf *out,
-	const struct lowdown_node *n, const struct lowdown_buf *in,
-	const struct sty *osty)
+    const struct lowdown_node *n, const struct lowdown_buf *in,
+    const struct sty *osty)
 {
 	size_t		 i = 0, len;
 	const char	*start;
@@ -822,9 +914,10 @@ rndr_buf(struct term *term, struct lowdown_buf *out,
 	const struct lowdown_node *n, const struct lowdown_buf *in,
 	const struct sty *osty)
 {
-	size_t				 i = 0, len, cols;
+	size_t				 i = 0, len, cols, nlen;
 	ssize_t				 ret;
-	int				 needspace, begin = 1, end = 0;
+	int				 needspace, hasspace,
+					 begin = 1, end = 0;
 	const char			*start;
 	const struct lowdown_node	*nn;
 
@@ -833,22 +926,30 @@ rndr_buf(struct term *term, struct lowdown_buf *out,
 	  	    nn->type == LOWDOWN_BLOCKHTML)
 			return rndr_buf_literal(term, out, n, in, osty);
 
-	/* Start each word by seeing if it has leading space. */
-
 	while (i < in->size) {
+		/*
+		 * Whether we need a space (word begins with space) and
+		 * have a space (current printed content ends with one).
+		 */
+
 		needspace = isspace((unsigned char)in->data[i]);
+		hasspace = out->size > 0 &&
+			isspace((unsigned char)out->data[out->size - 1]);
+
+		/* Skip to next word, then see how long the word is. */
 
 		while (i < in->size &&
 		       isspace((unsigned char)in->data[i]))
 			i++;
-
-		/* See how long it the coming word (may be 0). */
-
 		start = &in->data[i];
 		while (i < in->size &&
 		       !isspace((unsigned char)in->data[i]))
 			i++;
+
+		/* Get length and adjusted length (includes space). */
+
 		len = &in->data[i] - start;
+		nlen = len + (needspace ? 1 : 0);
 
 		/*
 		 * If we cross our maximum width and are preceded by a
@@ -859,10 +960,9 @@ rndr_buf(struct term *term, struct lowdown_buf *out,
 		 * This will also unset the current style.
 		 */
 
-		if ((needspace ||
-	 	     (out->size && isspace
-		      ((unsigned char)out->data[out->size - 1]))) &&
-		    term->col && term->col + len > term->maxcol) {
+		if ((needspace || hasspace) &&
+		    term->col > 0 &&
+		    term->col + nlen >= term->width) {
 			if (!rndr_buf_endline(term, out, n, osty))
 				return 0;
 			end = 0;
@@ -949,6 +1049,32 @@ rndr_entity(struct lowdown_buf *buf, int32_t val)
 	return 1;
 }
 
+/*
+ * Render a horizontal rule by repeating the character(s) in "hr" until
+ * the full screen width has been met.  This presumes that the caller
+ * has started a new line.  The style may be NULL.  A zero-length rule
+ * is silently ignored.
+ */
+static int
+rndr_hrule(struct term *st, struct lowdown_buf *ob, const char *hr,
+    const struct lowdown_node *n, const struct sty *sty)
+{
+	ssize_t	 ssz;
+	size_t	 sz, i;
+
+	if ((sz = strlen(hr)) == 0)
+		return 1;
+	if ((ssz = rndr_mbswidth(st, hr, sz)) < 0)
+		return 0;
+	if (ssz == 0)
+		return 1;
+	hbuf_truncate(st->tmp);
+	for (i = 0; i + ssz <= st->width; i += ssz)
+		if (!hbuf_puts(st->tmp, hr))
+			return 0;
+	return rndr_buf_literal(st, ob, n, st->tmp, sty);
+}
+
 /*
  * Adjust the stack of current nodes we're looking at.
  */
@@ -1024,12 +1150,12 @@ rndr_table(struct lowdown_buf *ob, struct term *st,
 				 * line wrapping.
 				 */
 
-				maxcol = st->maxcol;
+				maxcol = st->width;
 				last_blank = st->last_blank;
 				col = st->col;
 
 				st->last_blank = 0;
-				st->maxcol = SIZE_MAX;
+				st->width = SIZE_MAX;
 				st->col = 1;
 				if (!rndr(celltmp, st, cell))
 					goto out;
@@ -1037,7 +1163,7 @@ rndr_table(struct lowdown_buf *ob, struct term *st,
 					widths[i] = st->col;
 				st->last_blank = last_blank;
 				st->col = col;
-				st->maxcol = maxcol;
+				st->width = maxcol;
 			}
 	}
 
@@ -1057,12 +1183,12 @@ rndr_table(struct lowdown_buf *ob, struct term *st,
 			TAILQ_FOREACH(cell, &row->children, entries) {
 				i = cell->rndr_table_cell.col;
 				hbuf_truncate(celltmp);
-				maxcol = st->maxcol;
+				maxcol = st->width;
 				last_blank = st->last_blank;
 				col = st->col;
 
 				st->last_blank = 0;
-				st->maxcol = SIZE_MAX;
+				st->width = SIZE_MAX;
 				st->col = 1;
 				if (!rndr(celltmp, st, cell))
 					goto out;
@@ -1104,14 +1230,16 @@ rndr_table(struct lowdown_buf *ob, struct term *st,
 
 				st->last_blank = last_blank;
 				st->col = col;
-				st->maxcol = maxcol;
+				st->width = maxcol;
 
 				if (TAILQ_NEXT(cell, entries) == NULL)
 					continue;
 
-				if (!rndr_buf_style(st, rowtmp, &sty_table) ||
-				    !hbuf_printf(rowtmp, " %s ", ifx_table_col) ||
-				    !rndr_buf_unstyle(st, rowtmp, &sty_table))
+				if (!HBUF_PUTSL(rowtmp, " ") ||
+				    !rndr_buf_style(st, rowtmp, &sty_tbl) ||
+				    !hbuf_puts(rowtmp, ifx_tbl_col) ||
+				    !rndr_buf_unstyle(st, rowtmp, &sty_tbl) ||
+				    !HBUF_PUTSL(rowtmp, " "))
 					goto out;
 			}
 
@@ -1144,19 +1272,19 @@ rndr_table(struct lowdown_buf *ob, struct term *st,
 			st->stackpos++;
 			if (!rndr_stackpos_init(st, n))
 				goto out;
-			if (!rndr_buf_startline(st, ob, n, &sty_table))
+			if (!rndr_buf_startline(st, ob, n, &sty_tbl))
 				goto out;
 			for (i = 0; i < n->rndr_table.columns; i++) {
 				for (j = 0; j < widths[i]; j++)
-					if (!hbuf_puts(ob, ifx_table_row))
+					if (!hbuf_puts(ob, ifx_tbl_row))
 						goto out;
 				if (i < n->rndr_table.columns - 1 &&
-				    !hbuf_printf(ob, "%s%s",
-				    ifx_table_col, ifx_table_row))
+				    !hbuf_puts(ob, ifx_tbl_mcol) &&
+				    !hbuf_puts(ob, ifx_tbl_row))
 					goto out;
 			}
 			rndr_buf_advance(st, 1);
-			if (!rndr_buf_endline(st, ob, n, &sty_table))
+			if (!rndr_buf_endline(st, ob, n, &sty_tbl))
 				goto out;
 			if (!rndr_buf_vspace(st, ob, n, 1))
 				goto out;
@@ -1273,62 +1401,6 @@ rndr_doc_header(struct lowdown_buf *ob, struct term *st,
 	return 1;
 }
 
-/*
- * Extract metadata into "metaq".  This avoids calling rndr_buf()
- * because of all the line fiddling: because we know the contents of the
- * LOWDOWN_META, serialise directly into the metadata value without
- * formatting.
- *
- * This will be serialised to the output buffer in rndr_doc_header().
- *
- * Return zero on failure (memory), non-zero otherwise.
- */
-static int
-rndr_meta(struct term *st, const struct lowdown_node *n)
-{
-	struct lowdown_meta		*m;
-	const struct lowdown_node	*child;
-	struct lowdown_buf		*metatmp;
-	int32_t				 entity;
-
-	m = calloc(1, sizeof(struct lowdown_meta));
-	if (m == NULL)
-		return 0;
-	TAILQ_INSERT_TAIL(&st->metaq, m, entries);
-	m->key = strndup(n->rndr_meta.key.data,
-		n->rndr_meta.key.size);
-	if (m->key == NULL)
-		return 0;
-	if ((metatmp = hbuf_new(128)) == NULL)
-		return 0;
-	TAILQ_FOREACH(child, &n->children, entries) {
-		switch (child->type) {
-		case LOWDOWN_NORMAL_TEXT:
-			if (!hbuf_putb(metatmp,
-			    &child->rndr_normal_text.text))
-				return 0;
-			break;
-		case LOWDOWN_ENTITY:
-			entity = entity_find_iso
-				(&child->rndr_entity.text);
-			if (entity == 0)
-				break;
-			hbuf_truncate(st->tmp);
-			if (!rndr_entity(st->tmp, entity) ||
-			    !hbuf_putb(metatmp, st->tmp))
-				return 0;
-			break;
-		default:
-			abort();
-		}
-	}
-	m->value = strndup(metatmp->data, metatmp->size);
-	if (m->value == NULL)
-		return 0;
-	hbuf_free(metatmp);
-	return 1;
-}
-
 static int
 rndr(struct lowdown_buf *ob, struct term *st,
 	const struct lowdown_node *n)
@@ -1382,10 +1454,12 @@ rndr(struct lowdown_buf *ob, struct term *st,
 		vs = n->rndr_math.blockmode ? 1 : 0;
 		break;
 	case LOWDOWN_DEFINITION_DATA:
-	case LOWDOWN_HRULE:
 	case LOWDOWN_LINEBREAK:
 		vs = 1;
 		break;
+	case LOWDOWN_HRULE:
+		vs = 2;
+		break;
 	case LOWDOWN_LISTITEM:
 		vs = 1;
 		if (n->rndr_listitem.flags & HLIST_FL_BLOCK) {
@@ -1406,6 +1480,11 @@ rndr(struct lowdown_buf *ob, struct term *st,
 	/* Output leading content. */
 
 	switch (n->type) {
+	case LOWDOWN_IMAGE:
+	case LOWDOWN_LINK:
+	case LOWDOWN_LINK_AUTO:
+		st->in_link = n;
+		break;
 	case LOWDOWN_SUPERSCRIPT:
 		hbuf_truncate(st->tmp);
 		if (!hbuf_puts(st->tmp, ifx_super) ||
@@ -1450,7 +1529,7 @@ rndr(struct lowdown_buf *ob, struct term *st,
 			return 0;
 		break;
 	case LOWDOWN_META:
-		if (!rndr_meta(st, n))
+		if (lowdown_get_meta(n, &st->metaq) == NULL)
 			return 0;
 		break;
 	default:
@@ -1471,10 +1550,7 @@ rndr(struct lowdown_buf *ob, struct term *st,
 			return 0;
 		break;
 	case LOWDOWN_HRULE:
-		hbuf_truncate(st->tmp);
-		if (!hbuf_puts(st->tmp, ifx_hrule))
-			return 0;
-		if (!rndr_buf(st, ob, n, st->tmp, NULL))
+		if (!rndr_hrule(st, ob, ifx_hrule, n, NULL))
 			return 0;
 		break;
 	case LOWDOWN_FOOTNOTE:
@@ -1534,13 +1610,27 @@ rndr(struct lowdown_buf *ob, struct term *st,
 		}
 		break;
 	case LOWDOWN_LINK:
-		if (st->opts & LOWDOWN_TERM_NOLINK)
+		/*
+		 * The child content of the link has already been
+		 * produced to the output buffer.  Inhibit printing the
+		 * link address if requested for all links or if a
+		 * relative address and requested only for those.
+		 */
+
+		if ((st->opts & LOWDOWN_TERM_NOLINK) ||
+		    ((st->opts & LOWDOWN_TERM_NORELLINK) &&
+		     link_isrelative(&n->rndr_link.link)))
 			break;
+
+		/* Separate between text and link address. */
+
 		hbuf_truncate(st->tmp);
-		if (!HBUF_PUTSL(st->tmp, " "))
-			return 0;
-		if (!rndr_buf(st, ob, n, st->tmp, NULL))
+		if (!hbuf_puts(st->tmp, ifx_link_sep) ||
+		    !rndr_buf(st, ob, n, st->tmp, NULL))
 			return 0;
+
+		/* Format the link address. */
+
 		if (st->opts & LOWDOWN_TERM_SHORTLINK) {
 			hbuf_truncate(st->tmp);
 			if (!hbuf_shortlink
@@ -1555,91 +1645,109 @@ rndr(struct lowdown_buf *ob, struct term *st,
 		}
 		break;
 	case LOWDOWN_IMAGE:
-		if (!rndr_buf(st, ob, n, &n->rndr_image.alt, NULL))
+		/*
+		 * This is a bit more complicated than LOWDOWN_LINK
+		 * because the image "alt" is in a buffer and not
+		 * arranged as child nodes.  Begin with the image-left
+		 * bracketing.
+		 */
+
+		hbuf_truncate(st->tmp);
+		if (!hbuf_puts(st->tmp, ifx_imgbox_left) ||
+		    !rndr_buf(st, ob, n, st->tmp, &sty_imgbox) ||
+		    !rndr_buf(st, ob, n, &n->rndr_image.alt,
+		    &sty_linkalt))
 			return 0;
-		if (n->rndr_image.alt.size) {
+
+		/* If omitting the link, right-bracket and bail. */
+
+		if ((st->opts & LOWDOWN_TERM_NOLINK) ||
+		    ((st->opts & LOWDOWN_TERM_NORELLINK) &&
+		     link_isrelative(&n->rndr_image.link))) {
 			hbuf_truncate(st->tmp);
-			if (!HBUF_PUTSL(st->tmp, " "))
-				return 0;
-			if (!rndr_buf(st, ob, n, st->tmp, NULL))
-				return 0;
-		}
-		if (st->opts & LOWDOWN_TERM_NOLINK) {
-			hbuf_truncate(st->tmp);
-			if (!hbuf_puts(st->tmp, ifx_imgbox_left))
-				return 0;
-			if (!hbuf_puts(st->tmp, ifx_imgbox_right))
-				return 0;
-			if (!rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
+			if (!hbuf_puts(st->tmp, ifx_imgbox_right) ||
+			    !rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
 				return 0;
 			break;
 		}
+
+		/* Separate between text and link address. */
+
 		hbuf_truncate(st->tmp);
-		if (!hbuf_puts(st->tmp, ifx_imgbox_left))
-			return 0;
-		if (!hbuf_puts(st->tmp, ifx_imgbox_sep))
-			return 0;
-		if (!rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
+		if (!hbuf_puts(st->tmp, ifx_imgbox_sep) ||
+		    !rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
 			return 0;
+
+		/* Format link address. */
+
 		if (st->opts & LOWDOWN_TERM_SHORTLINK) {
-			hbuf_truncate(st->tmp);
 			if (!hbuf_shortlink
 			    (st->tmp, &n->rndr_image.link))
 				return 0;
 			if (!rndr_buf(st, ob, n, st->tmp, &sty_imgurl))
 				return 0;
 		} else
-			if (!rndr_buf(st, ob, n,
-			    &n->rndr_image.link, &sty_imgurl))
+			if (!rndr_buf(st, ob, n, &n->rndr_image.link,
+			    &sty_imgurl))
 				return 0;
+
+		/* Right-bracket and end. */
+
 		hbuf_truncate(st->tmp);
-		if (!hbuf_puts(st->tmp, ifx_imgbox_right))
-			return 0;
-		if (!rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
+		if (!hbuf_puts(st->tmp, ifx_imgbox_right) ||
+		    !rndr_buf(st, ob, n, st->tmp, &sty_imgbox))
 			return 0;
 		break;
 	case LOWDOWN_NORMAL_TEXT:
-		if (!rndr_buf(st, ob, n,
-		     &n->rndr_normal_text.text, NULL))
+		if (!rndr_buf(st, ob, n, &n->rndr_normal_text.text,
+		    NULL))
 			return 0;
 		break;
 	default:
 		break;
 	}
 
-	/* Trailing block spaces. */
+	switch (n->type) {
+	case LOWDOWN_IMAGE:
+	case LOWDOWN_LINK:
+	case LOWDOWN_LINK_AUTO:
+		st->in_link = NULL;
+		break;
+	case LOWDOWN_ROOT:
+		/*
+		 * If there are footnotes, begin by offsetting with
+		 * vertical space.  Then, if there's a footnote block
+		 * header, output that followed by vertical space.
+		 * Lastly, output the footnotes themselves.
+		 */
 
-	if (n->type == LOWDOWN_ROOT) {
+		if (st->footsz && !rndr_buf_vspace(st, ob, n, 2))
+			return 0;
 		if (st->footsz) {
-			if (!rndr_buf_vspace(st, ob, n, 2))
-				return 0;
-			hbuf_truncate(st->tmp);
-			if (!hbuf_puts(st->tmp, pfx_body.text))
-				return 0;
-			if (!hbuf_puts(st->tmp, ifx_foot))
-				return 0;
-			if (!rndr_buf_literal(st, ob, n, st->tmp,
-			    &sty_foot))
+			if (!rndr_hrule(st, ob, ifx_foot, n, &sty_foot))
 				return 0;
 			if (!rndr_buf_vspace(st, ob, n, 2))
 				return 0;
-			for (i = 0; i < st->footsz; i++)
-				if (!hbuf_putb(ob, st->foots[i]) ||
-				    !HBUF_PUTSL(ob, "\n"))
-					return 0;
 		}
+		for (i = 0; i < st->footsz; i++)
+			if (!hbuf_putb(ob, st->foots[i]) ||
+			    !HBUF_PUTSL(ob, "\n"))
+				return 0;
 		if (!rndr_buf_vspace(st, ob, n, 1))
 			return 0;
+		
+		/* Strip trailing newlines but for the vmargin. */
+
 		while (ob->size && ob->data[ob->size - 1] == '\n')
 			ob->size--;
 		if (!HBUF_PUTSL(ob, "\n"))
 			return 0;
-
-		/* Strip breaks but for the vmargin. */
-
 		for (i = 0; i < st->vmargin; i++)
 			if (!HBUF_PUTSL(ob, "\n"))
 				return 0;
+		break;
+	default:
+		break;
 	}
 
 	return 1;
@@ -1654,6 +1762,7 @@ lowdown_term_rndr(struct lowdown_buf *ob,
 
 	TAILQ_INIT(&st->metaq);
 	st->stackpos = 0;
+	st->in_link = NULL;
 	rc = rndr(ob, st, n);
 	rndr_free_footnotes(st);
 	lowdown_metaq_free(&st->metaq);
@@ -1668,15 +1777,43 @@ lowdown_term_new(const struct lowdown_opts *opts)
 	if ((st = calloc(1, sizeof(struct term))) == NULL)
 		return NULL;
 
-	/* Give us 80 columns by default. */
-
 	if (opts != NULL) {
-		st->maxcol = opts->cols == 0 ? 80 : opts->cols;
-		st->hmargin = opts->hmargin;
-		st->vmargin = opts->vmargin;
+		/*
+		 * Compute the width of the content pre-padding.  If
+		 * zero, limit to 80 or the number of terminal columns.
+		 * Otherwise, truncate to the number of columns.
+		 */
+
+		if (opts->term.width == 0) {
+			if ((st->width = opts->term.cols) > 80)
+				st->width = 80;
+		} else if (opts->term.width > opts->term.cols) {
+			st->width = opts->term.cols;
+		} else
+			st->width = opts->term.width;
+
+		/*
+		 * Compute the horizontal margin: either as given or, if
+		 * centred, computed from the content width.
+		 */
+
+		if (opts->term.centre && st->width < opts->term.cols)
+			st->hmargin = (opts->term.cols - st->width) / 2;
+		else
+			st->hmargin = opts->term.hmargin;
+
+		st->hpadding = opts->term.hpadding;
+		st->vmargin = opts->term.vmargin;
 		st->opts = opts->oflags;
-	} else
-		st->maxcol = 80;
+	} else {
+		st->width = 80;
+		st->hpadding = 4;
+	}
+
+	if (st->hpadding >= st->width)
+		st->width = 1;
+	else
+		st->width -= st->hpadding;
 
 	if ((st->tmp = hbuf_new(32)) == NULL) {
 		free(st);
